function [z_out, p_out, b_out, a_out] = allpass(b,a)

% Input: b, a = numerator and denominator of H(z)
% Output: z_out, p_out, b_out, a_out = zeros, poles, numerator, 
% denominator of the allpass transfer function related to H(z)

%%  first possibility: create Hap as H(z) / tilde{H(z)}

% do this for both numerator and denominator
a_tilde = fliplr(conj(a));
b_tilde = fliplr(conj(b));

b_out = conv(b, a_tilde);
a_out = conv(a, b_tilde);

% usually, every filter is normalized such that a_0 = 1 
% (but it's just a scaling operation)
b_out = b_out / a_out(1);
a_out = a_out / a_out(1);

z_out = roots(b_out);
p_out = roots(a_out);

%% second possibility: extract the zeros and the poles of H(z)
% put other zeros and poles in conjugate reciprocal position.

zeroes = roots(b);
poles = roots(a);

% initialize the zeros and poles of the output allpass filter
z_out = zeroes;
p_out = poles;

% loop over poles
for p = poles
    
    % check zeros
    % if there is already one zero which is the reciprocal conjugate of the
    % pole, go on to check the next pole
    % ideally:
    % if any(zeroes == 1/conj(p))
    if any((zeroes-1/conj(p)).^2 <= 1e-6)
        continue
    else
        % otherwise, add one zero = reciprocal conjugate of the pole
        z_out = [z_out; 1/conj(p)];
    end
    
end

% loop over zeros
for z = zeroes
    
    % check poles
    % if there is already one pole which is the reciprocal conjugate of the
    % zero, go on to check the next zero
    if any((poles-1/conj(z)).^2 <= 1e-6)
        continue
    else
        % otherwise, add one pole = reciprocal conjugate of the zero
        p_out = [p_out; 1/conj(z)];
    end
end

% create the related polynomials in z^-1
b_out = poly(z_out);
a_out = poly(p_out);

% remember to compute the value of c0!!
c_0 = sum(a_out) / sum(b_out);

% multiply the numerator b_out by c_0
b_out = c_0 * b_out;

end
